// --- [ ir_ma_transmit.c ] 18T ----------------------------------------------
//       tab = 3, 24.01.09
// Routine zum Senden von 2 Byte via Manchester-Codierung auf 36 KHz
// nicht kompatibel mit RC5-Standard !!

// Startsequenz: ein Impuls mit 1/2 Taktlnge,
// an der folgenden steigenden Flanke wird die Zeit synchronisiert,
// es folgt dann noch eine halbe Bitzeit low, dann folgen die Daten

// Das linke Bit wird getestet:
// - ist es eine 1, dann wird eine low/high Sequenz gesendet,
// - ist es eine 0, dann wird eine high/low Sequenz gesendet,
// danach wird das Word nach links geschoben.
// die Dauer eines Bits betrgt 1.0ms, das sind 18 cycles bei 36KHz low sowie
// 18 cycles high: (18 + 18) / 36000 = 36 / 36000 = 1000us (2x 500us)

#include <avr/io.h>
#include <inttypes.h>
#include <util/delay.h>
#include <stdlib.h>
#include "ir_master.h"

// --- static Function
void IR_ma_transmit(uint8_t data);

// wenn die LED nach Plus (highside) geschaltet wird, aktivieren, bei lowside auskommentieren
// sorgt dafr, dass nach dem Start die LED in der ersten Impulshlfe eingeschaltet ist ...
// ist vermutlich berflssig ...

//#define HIGHSIDE

// ----------------------------------------------------------------------------
// sendet 'out' und wartet auf die Antwort des Slave  
// liefert die Antwort  (bzw. einen Fehlercode) zurck
uint8_t IR_send(uint8_t out)
{
uint8_t in;	
uint8_t j;
							
IR_ma_transmit(out);							// Wartezeit vor dem Senden wird in aufrufender Routine definiert						

_delay_ms(1);									// Warten bis der Empfnger nach die eigene Sendung wieder auf Ruhepegel ist
IR_init();										// den Empfnger initialisieren 

for (j = 0; j< 20; j++) 					// bis zu 20ms auf Antwort warten
	{
	_delay_ms(1);								// 250x _delay_us(90) funktioniert nicht !
	if (IR_complete()) break;				// die Wartezeit stimmt, aber der Slave antwortet nicht ???
	}

IR_rx_off();									// Empfang deaktivieren

if (IR_complete())							// wenn eine Antwort eingegangen ist
	{
	in = IR_read();							// Antwort einlesen und die Adressen vergleichen
	if ((out & 0xF8) == (in & 0xF8)) 	// die Adressen stimmen berein
		{
		in &= 0x07;								// Slave-Status ohne Adresse zurckgeben
		}

	else								
		{
		in = 254;								// zurckgegebene Adresse stimmt nicht mit Aufruf berein
		}	
	}
	
else												// if !(IR_complete())
	{
	in = 255; 									// Fehlercode: keine Antwort
	}
return (in);									// in wird in den TWI_Ausgangspuffer geschrieben

}

// ----------------------------------------------------------------------------
// Wartet einen halben Takt 
// _delay_us kann nur bit 768us/MHz verzgern, somit sind bei 8 MHz max 96us erreichbar
// Dauer = 1/36000 * Impulse = 500us
void wait_hT(void)
{
register uint8_t i;
for(i = 0; i < 10; i++)
	{
	_delay_us(50);								// 10 * 50 entspricht 500us
	}
}

// ----------------------------------------------------------------------------
// sendet 1 Startbit und anschieend (sendbyte * 256 + ~sendbyte) = 16Bit
void IR_ma_transmit(uint8_t sendbyte)
{
register uint8_t count;
uint16_t data;
													// Empfang stoppen
TIMSK  = 0;										// Timer0 Interrupt abschalten
TCCR0B = 0;										// Timer0 stoppen
GIMSK  = 0;										// INT0 Interrupt abschalten

TCCR0A = IR_CONFIG;							// Register fr die Frequenzerzeugung setzen
OCR0A  = COMPARE;								// Comparewert fr toggle OC0A

data = sendbyte << 8;						// ins highbyte der 16bit Variablen schieben
data += ((uint8_t) (~sendbyte));			// und invertiertes Daten ins lowbyte kopieren

// Startbit: 1/2 Takte auf low (Senden), danach 1/2 Takt auf high(Pause)
													// 1. Taktteil LED aktiv
		TCNT0 = 0;								// Zhler zurcksetzen
#ifdef HIGHSIDE
		IR_PORT &= (~(1<<IR_PIN));			// LED einschalten (auf low setzen)
#else
		IR_PORT |= (1<<IR_PIN);				// LED einschalten (auf high setzen)
#endif
		DDR_IR_PORT |= (1<<IR_PIN);		// Pin auf Ausgang

		TCCR0B = PWM_ON;						// PWM starten, 
		wait_hT();								// halben Takt senden

													// 2. Taktteil Pause
		TCCR0B = PWM_OFF;						// PWM stoppen -> der Empfnger beginnt hier mit der Zhlung
		DDR_IR_PORT &= (~(1<<IR_PIN));	// DDR auf Eingang
#ifdef HIGHSIDE
		IR_PORT |= (1<<IR_PIN);				// LED ausschalten (auf high setzen)
#else
		IR_PORT &= (~(1<<IR_PIN));			// LED ausschalten (auf low setzen)
#endif
		wait_hT();
													// Jetzt folgen die Daten

for(count = 0; count < IR_BITS; count++)// Anzahl der zu sendenden Bits regelt count
	{
	if (data & (1<<(IR_BITS - 1)))		// wenn das 'linkste' Bit gesetzt ist, dann
		{											// eine '1' ausgeben, erst low, dann high

													// 1. Takthlfte low
		TCCR0B = PWM_OFF;						// PWM stoppen
		DDR_IR_PORT &= (~(1<<IR_PIN));	// DDR auf Eingang

#ifdef HIGHSIDE
		IR_PORT |= (1<<IR_PIN);				// LED ausschalten (auf high setzen)
#else
		IR_PORT &= (~(1<<IR_PIN));			// LED ausschalten (auf low setzen)
#endif

		wait_hT();
													// 2. Takthlfte high
		TCNT0 = 0;								// Zhler zurcksetzen
#ifdef HIGHSIDE
		IR_PORT &= (~(1<<IR_PIN));			// LED einschalten (auf low setzen)
#else
		IR_PORT |= (1<<IR_PIN);				// LED einschalten (auf high setzen)
#endif
		DDR_IR_PORT |= (1<<IR_PIN);		// Pin auf Ausgang
		TCCR0B = PWM_ON;						// PWM starten
		wait_hT();								// PWM luft beim Verlassen noch !!
		}
	else											// wenn low-Byte von data  = 0, dann
		{											// eine '0' senden, erst high, dann low

		TCNT0 = 0;								// 1. Takthlfte high
#ifdef HIGHSIDE
		IR_PORT &= (~(1<<IR_PIN));			// LED einschalten (auf low setzen)
#else
		IR_PORT |= (1<<IR_PIN);				// LED einschalten (auf high setzen)
#endif
		DDR_IR_PORT |= (1<<IR_PIN);		// Portpin auf Ausgang
		TCCR0B = PWM_ON;						// den PWM starten
		wait_hT();
													// 2. Takthlfte low
		TCCR0B = PWM_OFF;
		DDR_IR_PORT &= (~(1<<IR_PIN));	// Portpin als Eingang schalten

#ifdef HIGHSIDE
		IR_PORT |= (1<<IR_PIN);				// LED ausschalten (auf high setzen)
#else
		IR_PORT &= (~(1<<IR_PIN));			// LED ausschalten (auf low setzen)
#endif
		wait_hT();
		}

	data = data << 1;							// Bits nach links schieben
	}												// for i < anzahl_bits

	TCCR0B = PWM_OFF;							// PWM abschalten
	DDR_IR_PORT &= (~(1<<IR_PIN));		// den IR_Pin als Eingang schalten
	IR_PORT &= (~(1<<IR_PIN));				// Portpin auf low, Pullup aus
}

// --- [ end of file ] --------------------------------------------------------