//--- [ ir_slave.c ] 18T ------------------------------------------------------
//      tab = 3, 24.01.09
// 
// IR-Empfnger, Trger 36KHz, Impulsbreite 500 + 500us (18 Impulse)
// 1000 Bit /Sekunde
// benutzt den Timer0 fr die Zeitbestimmung bei Empfang
// liefert eine Quittung an den Master zurck, 
// sofern gltige Daten und gltige Adresse empfangen wurde - sonst schweigt der slave
//
// wichtig: CPU mit RC-Oscillator einsetzen - oder Wartezeiten beim Sleep ndern:
// beim Sender auf 5ms erhhen
// beim Empfnger wait1ms entfernen (entsteht automatisch durch langsamen Start aus sleep)
// ----------------------------------------------------------------------------

#include <avr/io.h>
#include <inttypes.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <stdlib.h>
#include <avr/sleep.h>
#include <avr/eeprom.h>
#include "ir_slave.h"
	
uint8_t eemem[3] EEMEM;						// eemem[0] = IR-Slave-Adr, 
													// eemem[1] = OSCCAL
													// eemem[2] = Kalibriermodus (wenn == 1)
uint8_t ir_slave_adr; 

// ----------------------------------------------------------------------------
// gibt an OC0A fr 15 Sekunden ein 36kHz - Signal aus
// durch Messung der Frequenz und Korrektur des OSCCAL-Wertes (im eeprom) 
// kann auf die Sollfrequenz abgestimmt werden

void kalibrieren(void)
{
uint8_t i,j;

DDR_IR_PORT |= (1<<IR_PIN);				// Sendepin als Ausgang schalten
OCR0A  = COMPARE;								// Comparewert fr toggle OC0A
TCCR0A = IR_CONFIG;							// Register fr die Frequenzerzeugung setzen
TCCR0B = PWM_ON;								// PWM starten, 

j = 15;											// 30 * 1Sek
while(j)
	{
	i = 40;										// 40 * 25ms = 1000ms
	while(i)
		{
		_delay_ms(25);							// max. errreichbare Zeit = 262ms / F_CPU
		i--;
		}
	j--;
	}
	
TCCR0B = PWM_OFF;								// PWM abschalten
DDR_IR_PORT &= ~(1<<IR_PIN);				// Sendepin als Eingang schalten
IR_PORT &= ~(1<<IR_PIN);					// Sendepin auf 0
}

// ----------------------------------------------------------------------------
int main(void)
{
uint16_t data = 0;
uint8_t highbyte, lowbyte, j;
														// Oszillator luft mit 8MHz, Vorteiler 1 fr 8MHz
CLKPR = (1<<CLKPCE);								// 0b10000000 schreiben
CLKPR = (0<<CLKPS0);								// danach sofort den Teiler setzen

j = eeprom_read_byte(&eemem[0]);				// die Slave-Adresse ggf. aus eeprom lesen
														// und gleich um 3 Stellen nach links schieben
if (j < 32) ir_slave_adr = (j * 8); else ir_slave_adr = (IR_ADRESS * 8); 

														// zum Kalibrieren des Systemtaktes kann ein OSCCAL-Wert
														// ins Eeprom an Pos. 2 geschrieben werden
j = eeprom_read_byte(&eemem[1]);				// wenn er grer als 50 und kleiner als 150 ist,
														// dann wird er bernommen.
if (j < 150 && j > 50) OSCCAL = j; else OSCCAL = OSCCAL_DEFAULT;

j = eeprom_read_byte(&eemem[2]);			// wenn hier eine 1 steht, dann in den Kalibriermodus wechseln
if (j == 1) kalibrieren();

// Ports
// PB.0 = OC0A   = Sender, 
// PB.4 = PCINT0 = Empfnger
// PB.1 / PB.2 / PB.3 = Schaltpin
PORTB = (1<<SWITCH1 | 1<<SWITCH2 | 1<<SWITCH3);	// Schaltpin auf high legen - nur bei HIGHSIDE
DDRB  = (1<<SWITCH1 | 1<<SWITCH2 | 1<<SWITCH3);	  

// Energieeinsparung
ACSR = (1<<ACD);
PRR  = (1<<PRTIM1 | 1<<PRUSI | 1<< PRADC);// Achtung: der Timer1 ist abgeschaltet !!

IR_init();											// der Slave muss im Empfangsmodus starten

while(1)												// Endlosschleife
	{
	if (IR_complete())							// wenn ein fehlerfreier Empfang vorliegt
		{
		data     = IR_read();					// Daten einlesen
		highbyte = (uint8_t) (data / 256);	// 16 Bit aufteilen auf 2 Byte
		lowbyte  = (uint8_t) data;
		lowbyte  = ~lowbyte;						// das low-Byte invertieren
														// wenn low-Byte = high-Byte, dann
		if (highbyte == lowbyte)				// wenn gleich, dann sind die Daten gltig
			{
														// die Adresse ist um 3 Bit nach links geschoben, daher (* 8)
			if ((highbyte & 0xF8) == ir_slave_adr)// wenn dann auch noch die Adresse (Bit 7-3) stimmt
				{
				if (highbyte & 0x07)				// wenn mindestens 1 toggle-Bits gesetzt ist,  
					{									// dann den Pinstatus ggf. togglen
					if (highbyte & 0x01)			// wenn Bit.0 gesetzt ist
						{
						PINB = (1<<SWITCH1);		// Pin fr Kanal 1 togglen
						}
					if (highbyte & 0x02)			// wenn Bit.0 gesetzt ist
						{
						PINB = (1<<SWITCH2);		// Pin fr Kanal 2 togglen
						}
					if (highbyte & 0x04)			// wenn Bit.0 gesetzt ist
						{
						PINB = (1<<SWITCH3);		// Pin fr Kanal 3 togglen
						}	
					}
					
				highbyte &= 0xF8;					// die toggle-Bits lschen
														// und den aktuellen Portstatus in die unteren 3 Bits schreiben
				if (PINB & (1<<SWITCH1))		// den Pinstatus von SWITCH1 zurckgeben
					{
					highbyte += 1;					// 1 addieren
					}
				if (PINB & (1<<SWITCH2))		// wenn gesetzt
					{
					highbyte += 2;					// den Pinstatus von SWITCH2 zurckgeben
					}
				if (PINB & (1<<SWITCH3))
					{
					highbyte += 4;					// den Pinstatus von SWITCH3 zurckgeben
					}
				

				_delay_ms(3);						// vor dem Senden kurze Wartezeit einlegen (3 ist besser)
				IR_sla_transmit(highbyte);		// Pinstatus an den Sender zurckgeben

				_delay_ms(1);						// Warten bis der Empfnger nach eigener Sendung wieder auf
														// auf Ruhepegel liegt
				IR_init();							// den Empfnger wieder initialisieren				
				}
			}											// Falsche Adresse
		}												// if (ir_complete())

// sleep erst dann aktivieren, wenn alles andere funktioniert
// und damit geklrt werden kann, ob das womglich sleep den Fehler produziert.

// sleep wird eingeleitet, wenn (50us * 255) Wartezeit um sind 
// und keine bertragung im Gange ist
// und kein Daten vollstndig empfangen wurden

	if ((!(IR_busy())) && (!(IR_complete())))
		{	
		j = MCUCR;
		MCUCR = (1<<SE | 1<<SM1);				// Sleep enable, Power-down Modus
		asm volatile ("sleep");					// wird durch PCINT0 beendet (TSOP geht auf 0)
		MCUCR = j;
		}
	}													// while (1)
}

// --- [ end of file ] --------------------------------------------------------
